/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

// E4QueryHome.java

package org.objectweb.jonas.jtests.beans.ebasic;

import java.rmi.RemoteException;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.EJBHome;
import javax.ejb.FinderException;

public interface E4QueryHome extends EJBHome {
    public E4Query create(String id, String s, int i, double d) throws RemoteException, CreateException;
    public E4Query findByPrimaryKey(String id) throws RemoteException, FinderException;
    public Collection findByLengthString(int l)
        throws RemoteException, FinderException;
    public Collection findByLocateString(String s)
        throws RemoteException, FinderException;
    public Collection findByLocateStringAt(String s, int start)
        throws RemoteException, FinderException;
    public Collection findBySubstring(String s, int start, int length)
        throws RemoteException, FinderException;
    public Collection findByConcatString(String s)
        throws RemoteException, FinderException;
    public Collection findByAbsInt(int i)
        throws RemoteException, FinderException;
    public Collection findBySqrtDouble(double d)
        throws RemoteException, FinderException;
    public Collection findByIsNull()
        throws RemoteException, FinderException;
    public Collection findByIsNullParam(String s)
        throws RemoteException, FinderException;
    public Collection findByInStrings()
        throws RemoteException, FinderException;
    public Collection findByLessThanMinus100()
        throws RemoteException, FinderException;
    public Collection findByIntEqualExpr(int i1, int i2, int i3)
        throws RemoteException, FinderException;
    public Collection findByIntEqualExpr2(int i1, int i2, int i3)
    throws RemoteException, FinderException;
    public Collection findByStringGreaterThenExpr(String s)
        throws RemoteException, FinderException;
    public Collection findByStringGreaterOrEqualThenExpr(String s)
        throws RemoteException, FinderException;
    public Collection findByIntModXIsZero(int i)
        throws RemoteException, FinderException;
}
